/**
 * A component of a library for
 * <a href="http://www.geog.leeds.ac.uk/people/a.turner/projects/MoSeS">MoSeS</a>.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 */
package uk.ac.leeds.ccg.andyt.projects.moses.io;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StreamTokenizer;
import java.util.HashMap;
import java.util.Iterator;
import java.util.TreeMap;
import uk.ac.leeds.ccg.andyt.projects.moses.process.GeneticAlgorithm_HSARHP_ISARCEP;
import uk.ac.leeds.ccg.andyt.projects.moses.utilities.StaticIO;

/**
 * Class for handling output that compares some optimisation constraint
 * variables in the CAS and IndividualCensus outputs.
 */
public class OutputDataHandler_OptimisationConstraints extends AbstractOutputDataHandler {

    public OutputDataHandler_OptimisationConstraints() {
    }

    /**
     * @param args
     *            the command line arguments
     */
    public static void main(String[] args) throws IOException {
        new OutputDataHandler_ControlConstraints().run();
    }

    public void run() throws IOException {
    }

    /**
     * Writes out header.
     */
    public void writeHeader() throws IOException {
        String header = new String(
                "_ZoneCode," + "_PeopleWhoseGeneralHealthWasGood,_PeopleWhoseGeneralHealthWasFairlyGood,_PeopleWhoseGeneralHealthWasNotGood,_PeopleWithLimitingLongTermIllness," + "_LoneParentHouseholdsWithChildren,_OneFamilyAndNoChildren,_MarriedOrCohabitingCoupleWithChildren," + "_FemalesAge0to4,_FemalesAge5to9,_FemalesAge10to14,_FemalesAge15to19,_FemalesAge20to24,_FemalesAge25to29,_FemalesAge30to34,_FemalesAge35to39,_FemalesAge40to44,_FemalesAge45to49,_FemalesAge50to54,_FemalesAge55to59,_FemalesAge60to64,_FemalesAge65to69,_FemalesAge70to74,_FemalesAge75to79,_FemalesAge80AndOver," + "_FemalesMarriedAge0to15,_FemalesMarriedAge16to19,_FemalesMarriedAge20to24,_FemalesMarriedAge25to29,_FemalesMarriedAge30to34,_FemalesMarriedAge35to39,_FemalesMarriedAge40to44,_FemalesMarriedAge45to49,_FemalesMarriedAge50to54,_FemalesMarriedAge55to59,_FemalesMarriedAge60to64,_FemalesMarriedAge65to74,_FemalesMarriedAge75to79,_FemalesMarriedAge80AndOver," + "_FemalesAge16to24Unemployed,_FemalesAge16to74,_FemalesAge16to74EconomicallyActiveEmployed,_FemalesAge16to74EconomicallyActiveUnemployed,_FemalesAge16to74EconomicallyInactive,_FemalesAge50AndOverUnemployed," + "_MalesAge0to4,_MalesAge5to9,_MalesAge10to14,_MalesAge15to19,_MalesAge20to24,_MalesAge25to29,_MalesAge30to34,_MalesAge35to39,_MalesAge40to44,_MalesAge45to49,_MalesAge50to54,_MalesAge55to59,_MalesAge60to64,_MalesAge65to69,_MalesAge70to74,_MalesAge75to79,_MalesAge80AndOver," + "_MalesMarriedAge0to15,_MalesMarriedAge16to19,_MalesMarriedAge20to24,_MalesMarriedAge25to29,_MalesMarriedAge30to34,_MalesMarriedAge35to39,_MalesMarriedAge40to44,_MalesMarriedAge45to49,_MalesMarriedAge50to54,_MalesMarriedAge55to59,_MalesMarriedAge60to64,_MalesMarriedAge65to74,_MalesMarriedAge75to79,_MalesMarriedAge80AndOver," + "_MalesAge16to24Unemployed,_MalesAge16to74,_MalesAge16to74EconomicallyActiveEmployed,_MalesAge16to74EconomicallyActiveUnemployed,_MalesAge16to74EconomicallyInactive,_MalesAge50AndOverUnemployed");
        System.out.println(header);
        this._FileOutputStream.write(header.getBytes());
        this._FileOutputStream.write(StreamTokenizer.TT_EOL);
        this._FileOutputStream.flush();
    }

    @Override
    public void writeObserved(
            String _CASDataDirectory,
            String _OutputFileName,
            long _StartRecordID,
            long _EndRecordID,
            String _AreaLevel,
            ISARDataHandler tISARDataHandler)
            throws IOException {
        this._OutputFile = new File(_OutputFileName);
        this._OutputFile.getParentFile().mkdir();
        // this._OutputFile = new File(
        // "C:/Work/Projects/MoSeS/Workspace/Leeds/MarkOutputConstraintsOA.csv"
        // );
        _FileOutputStream = new FileOutputStream(_OutputFile);
        this._CASDataHandler = new CASDataHandler(new File(_CASDataDirectory),
                "");
        // this._CASDataHandler = new CASDataHandler( new File(
        // "C:/Work/Projects/MoSeS/Workspace/" ), "" );
        writeHeader();
        CASDataRecord _CASDataRecord;
        CAS003DataRecord _CAS003DataRecord;
        CAS001DataRecord _CAS001DataRecord;
        Counts _Counts = new Counts();
        HashMap _CAS003AgeCountConstraintHashMap;
        HashMap _CAS001AgeCountHPConstraintHashMap;
        HashMap _CAS001AgeCountCEPConstraintHashMap;

        GeneticAlgorithm_HSARHP_ISARCEP _GeneticAlgorithm_HSARHP_ISARCEP;
        Object[] _FitnessCountsHSARHP;
        HashMap _CASCounts;

        for (long RecordID = _StartRecordID; RecordID <= _EndRecordID; RecordID++) {
            _CASDataRecord = (CASDataRecord) this._CASDataHandler.getDataRecord(RecordID);
            _GeneticAlgorithm_HSARHP_ISARCEP = new GeneticAlgorithm_HSARHP_ISARCEP(
                    _CASDataRecord);
            _FitnessCountsHSARHP = _GeneticAlgorithm_HSARHP_ISARCEP.getFitnessCounts();
            _CASCounts = (HashMap) _FitnessCountsHSARHP[0];
            // Health
            _Counts._PeopleWhoseGeneralHealthWasGood = (Integer) _CASCounts.get("peopleWhoseGeneralHealthWasGood");
            _Counts._PeopleWhoseGeneralHealthWasFairlyGood = (Integer) _CASCounts.get("peopleWhoseGeneralHealthWasFairlyGood");
            _Counts._PeopleWhoseGeneralHealthWasNotGood = (Integer) _CASCounts.get("peopleWhoseGeneralHealthWasNotGood");
            _Counts._PeopleWithLimitingLongTermIllness = (Integer) _CASCounts.get("peopleWithLimitingLongTermIllness");
            // Household Composition:
            _Counts._LoneParentHouseholdsWithChildren = (Integer) _CASCounts.get("loneParentHouseholdsWithChildren");
            _Counts._OneFamilyAndNoChildren = (Integer) _CASCounts.get("oneFamilyAndNoChildren");
            _Counts._MarriedOrCohabitingCoupleWithChildren = (Integer) _CASCounts.get("marriedOrCohabitingCoupleWithChildren");
            // Female
            _Counts._FemalesAge0to4 = (Integer) _CASCounts.get("femalesAge0to4");
            _Counts._FemalesAge5to9 = (Integer) _CASCounts.get("femalesAge5to9");
            _Counts._FemalesAge10to14 = (Integer) _CASCounts.get("femalesAge10to14");
            _Counts._FemalesAge15to19 = (Integer) _CASCounts.get("femalesAge15to19");
            _Counts._FemalesAge20to24 = (Integer) _CASCounts.get("femalesAge20to24");
            _Counts._FemalesAge25to29 = (Integer) _CASCounts.get("femalesAge25to29");
            _Counts._FemalesAge30to34 = (Integer) _CASCounts.get("femalesAge30to34");
            _Counts._FemalesAge35to39 = (Integer) _CASCounts.get("femalesAge35to39");
            _Counts._FemalesAge40to44 = (Integer) _CASCounts.get("femalesAge40to44");
            _Counts._FemalesAge45to49 = (Integer) _CASCounts.get("femalesAge45to49");
            _Counts._FemalesAge50to54 = (Integer) _CASCounts.get("femalesAge50to54");
            _Counts._FemalesAge55to59 = (Integer) _CASCounts.get("femalesAge55to59");
            _Counts._FemalesAge60to64 = (Integer) _CASCounts.get("femalesAge60to64");
            _Counts._FemalesAge65to69 = (Integer) _CASCounts.get("femalesAge65to69");
            _Counts._FemalesAge70to74 = (Integer) _CASCounts.get("femalesAge70to74");
            _Counts._FemalesAge75to79 = (Integer) _CASCounts.get("femalesAge75to79");
            _Counts._FemalesAge80AndOver = (Integer) _CASCounts.get("femalesAge80AndOver");
            _Counts._FemalesMarriedAge0to15 = (Integer) _CASCounts.get("femalesMarriedAge0to15");
            _Counts._FemalesMarriedAge16to19 = (Integer) _CASCounts.get("femalesMarriedAge16to19");
            _Counts._FemalesMarriedAge20to24 = (Integer) _CASCounts.get("femalesMarriedAge20to24");
            _Counts._FemalesMarriedAge25to29 = (Integer) _CASCounts.get("femalesMarriedAge25to29");
            _Counts._FemalesMarriedAge30to34 = (Integer) _CASCounts.get("femalesMarriedAge30to34");
            _Counts._FemalesMarriedAge35to39 = (Integer) _CASCounts.get("femalesMarriedAge35to39");
            _Counts._FemalesMarriedAge40to44 = (Integer) _CASCounts.get("femalesMarriedAge40to44");
            _Counts._FemalesMarriedAge45to49 = (Integer) _CASCounts.get("femalesMarriedAge45to49");
            _Counts._FemalesMarriedAge50to54 = (Integer) _CASCounts.get("femalesMarriedAge50to54");
            _Counts._FemalesMarriedAge55to59 = (Integer) _CASCounts.get("femalesMarriedAge55to59");
            _Counts._FemalesMarriedAge60to64 = (Integer) _CASCounts.get("femalesMarriedAge60to64");
            _Counts._FemalesMarriedAge65to74 = (Integer) _CASCounts.get("femalesMarriedAge65to74");
            _Counts._FemalesMarriedAge75to79 = (Integer) _CASCounts.get("femalesMarriedAge75to79");
            _Counts._FemalesMarriedAge80AndOver = (Integer) _CASCounts.get("femalesMarriedAge80AndOver");
            _Counts._FemalesAge16to24Unemployed = (Integer) _CASCounts.get("femalesMarriedAge80AndOver");
            _Counts._FemalesAge16to74 = (Integer) _CASCounts.get("femalesAge16to74");
            _Counts._FemalesAge16to74EconomicallyActiveEmployed = (Integer) _CASCounts.get("femalesAge16to74EconomicallyActiveEmployed");
            _Counts._FemalesAge16to74EconomicallyActiveUnemployed = (Integer) _CASCounts.get("femalesAge16to74EconomicallyActiveUnemployed");
            _Counts._FemalesAge16to74EconomicallyInactive = (Integer) _CASCounts.get("femalesAge16to74EconomicallyInactive");
            _Counts._FemalesAge50AndOverUnemployed = (Integer) _CASCounts.get("femalesAge50AndOverUnemployed");
            // Male
            _Counts._MalesAge0to4 = (Integer) _CASCounts.get("malesAge0to4");
            _Counts._MalesAge5to9 = (Integer) _CASCounts.get("malesAge5to9");
            _Counts._MalesAge10to14 = (Integer) _CASCounts.get("malesAge10to14");
            _Counts._MalesAge15to19 = (Integer) _CASCounts.get("malesAge15to19");
            _Counts._MalesAge20to24 = (Integer) _CASCounts.get("malesAge20to24");
            _Counts._MalesAge25to29 = (Integer) _CASCounts.get("malesAge25to29");
            _Counts._MalesAge30to34 = (Integer) _CASCounts.get("malesAge30to34");
            _Counts._MalesAge35to39 = (Integer) _CASCounts.get("malesAge35to39");
            _Counts._MalesAge40to44 = (Integer) _CASCounts.get("malesAge40to44");
            _Counts._MalesAge45to49 = (Integer) _CASCounts.get("malesAge45to49");
            _Counts._MalesAge50to54 = (Integer) _CASCounts.get("malesAge50to54");
            _Counts._MalesAge55to59 = (Integer) _CASCounts.get("malesAge55to59");
            _Counts._MalesAge60to64 = (Integer) _CASCounts.get("malesAge60to64");
            _Counts._MalesAge65to69 = (Integer) _CASCounts.get("malesAge65to69");
            _Counts._MalesAge70to74 = (Integer) _CASCounts.get("malesAge70to74");
            _Counts._MalesAge75to79 = (Integer) _CASCounts.get("malesAge75to79");
            _Counts._MalesAge80AndOver = (Integer) _CASCounts.get("malesAge80AndOver");
            _Counts._MalesMarriedAge0to15 = (Integer) _CASCounts.get("malesMarriedAge0to15");
            _Counts._MalesMarriedAge16to19 = (Integer) _CASCounts.get("malesMarriedAge16to19");
            _Counts._MalesMarriedAge20to24 = (Integer) _CASCounts.get("malesMarriedAge20to24");
            _Counts._MalesMarriedAge25to29 = (Integer) _CASCounts.get("malesMarriedAge25to29");
            _Counts._MalesMarriedAge30to34 = (Integer) _CASCounts.get("malesMarriedAge30to34");
            _Counts._MalesMarriedAge35to39 = (Integer) _CASCounts.get("malesMarriedAge35to39");
            _Counts._MalesMarriedAge40to44 = (Integer) _CASCounts.get("malesMarriedAge40to44");
            _Counts._MalesMarriedAge45to49 = (Integer) _CASCounts.get("malesMarriedAge45to49");
            _Counts._MalesMarriedAge50to54 = (Integer) _CASCounts.get("malesMarriedAge50to54");
            _Counts._MalesMarriedAge55to59 = (Integer) _CASCounts.get("malesMarriedAge55to59");
            _Counts._MalesMarriedAge60to64 = (Integer) _CASCounts.get("malesMarriedAge60to64");
            _Counts._MalesMarriedAge65to74 = (Integer) _CASCounts.get("malesMarriedAge65to74");
            _Counts._MalesMarriedAge75to79 = (Integer) _CASCounts.get("malesMarriedAge75to79");
            _Counts._MalesMarriedAge80AndOver = (Integer) _CASCounts.get("malesMarriedAge80AndOver");
            _Counts._MalesAge16to24Unemployed = (Integer) _CASCounts.get("malesMarriedAge80AndOver");
            _Counts._MalesAge16to74 = (Integer) _CASCounts.get("malesAge16to74");
            _Counts._MalesAge16to74EconomicallyActiveEmployed = (Integer) _CASCounts.get("malesAge16to74EconomicallyActiveEmployed");
            _Counts._MalesAge16to74EconomicallyActiveUnemployed = (Integer) _CASCounts.get("malesAge16to74EconomicallyActiveUnemployed");
            _Counts._MalesAge16to74EconomicallyInactive = (Integer) _CASCounts.get("malesAge16to74EconomicallyInactive");
            _Counts._MalesAge50AndOverUnemployed = (Integer) _CASCounts.get("malesAge50AndOverUnemployed");
            write(_Counts, String.valueOf(_CASDataRecord.getZone_Code()));
        }
    }

    /**
     * writeEstimated_HSARHP
     *
     * @param _Aggregation
     *            OA, MSOA, Ward
     */
    public void writeEstimated_HSARHP(String directory, String filenamePrefix,
            String filenameSuffix, String _Aggregation) throws IOException {
        this._OutputFile = new File(directory + filenamePrefix + filenameSuffix);
        this._OutputFile.createNewFile();
        File _InputFile = new File(directory + filenamePrefix + ".csv");
        writeEstimated_HSARHP(_InputFile, _OutputFile, _Aggregation);
    }

    /**
     * writeEstimated_HSARHP
     */
    public void writeEstimated_HSARHP(File _InputFile, File _OutputFile,
            String _Aggregation) throws IOException {
        ToyModelDataHandler tToyModelDataHandler = new ToyModelDataHandler();
        _OutputFile.getParentFile().mkdirs();
        this._FileOutputStream = new FileOutputStream(_OutputFile);
        writeHeader();
        CASDataRecord _CASDataRecord;
        int _PeopleWhoseGeneralHealthWasGood;
        int _PeopleWhoseGeneralHealthWasFairlyGood;
        int _PeopleWhoseGeneralHealthWasNotGood;
        int _PeopleWithLimitingLongTermIllness;
        int _LoneParentHouseholdsWithChildren;
        int _OneFamilyAndNoChildren;
        int _MarriedOrCohabitingCoupleWithChildren;
        int _FemalesAge0to4;
        int _FemalesAge5to9;
        int _FemalesAge10to14;
        int _FemalesAge15to19;
        int _FemalesAge20to24;
        int _FemalesAge25to29;
        int _FemalesAge30to34;
        int _FemalesAge35to39;
        int _FemalesAge40to44;
        int _FemalesAge45to49;
        int _FemalesAge50to54;
        int _FemalesAge55to59;
        int _FemalesAge60to64;
        int _FemalesAge65to69;
        int _FemalesAge70to74;
        int _FemalesAge75to79;
        int _FemalesAge80AndOver;
        int _FemalesMarriedAge0to15;
        int _FemalesMarriedAge16to19;
        int _FemalesMarriedAge20to24;
        int _FemalesMarriedAge25to29;
        int _FemalesMarriedAge30to34;
        int _FemalesMarriedAge35to39;
        int _FemalesMarriedAge40to44;
        int _FemalesMarriedAge45to49;
        int _FemalesMarriedAge50to54;
        int _FemalesMarriedAge55to59;
        int _FemalesMarriedAge60to64;
        int _FemalesMarriedAge65to74;
        int _FemalesMarriedAge75to79;
        int _FemalesMarriedAge80AndOver;
        int _FemalesAge16to24Unemployed;
        int _FemalesAge16to74;
        int _FemalesAge16to74EconomicallyActiveEmployed;
        int _FemalesAge16to74EconomicallyActiveUnemployed;
        int _FemalesAge16to74EconomicallyInactive;
        int _FemalesAge50AndOverUnemployed;
        int _MalesAge0to4;
        int _MalesAge5to9;
        int _MalesAge10to14;
        int _MalesAge15to19;
        int _MalesAge20to24;
        int _MalesAge25to29;
        int _MalesAge30to34;
        int _MalesAge35to39;
        int _MalesAge40to44;
        int _MalesAge45to49;
        int _MalesAge50to54;
        int _MalesAge55to59;
        int _MalesAge60to64;
        int _MalesAge65to69;
        int _MalesAge70to74;
        int _MalesAge75to79;
        int _MalesAge80AndOver;
        int _MalesMarriedAge0to15;
        int _MalesMarriedAge16to19;
        int _MalesMarriedAge20to24;
        int _MalesMarriedAge25to29;
        int _MalesMarriedAge30to34;
        int _MalesMarriedAge35to39;
        int _MalesMarriedAge40to44;
        int _MalesMarriedAge45to49;
        int _MalesMarriedAge50to54;
        int _MalesMarriedAge55to59;
        int _MalesMarriedAge60to64;
        int _MalesMarriedAge65to74;
        int _MalesMarriedAge75to79;
        int _MalesMarriedAge80AndOver;
        int _MalesAge16to24Unemployed;
        int _MalesAge16to74;
        int _MalesAge16to74EconomicallyActiveEmployed;
        int _MalesAge16to74EconomicallyActiveUnemployed;
        int _MalesAge16to74EconomicallyInactive;
        int _MalesAge50AndOverUnemployed;
        HSARDataHandler tHSARDataHandler = new HSARDataHandler(
                new File("C:/Work/Projects/MoSeS/Workspace/",
                "uk.ac.leeds.ccg.andyt.projects.moses.io.HSARDataHandler.thisFile"));
        ISARDataHandler tISARDataHandler = new ISARDataHandler(
                new File("C:/Work/Projects/MoSeS/Workspace/",
                "uk.ac.leeds.ccg.andyt.projects.moses.io.ISARDataHandler_AGE0Indexed.thisFile"));
        HSARDataRecord _HSARDataRecord;
        ISARDataRecord _ISARDataRecord;
        BufferedReader tBufferedReader = new BufferedReader(
                new InputStreamReader(new FileInputStream(_InputFile)));
        StreamTokenizer tStreamTokenizer = new StreamTokenizer(tBufferedReader);
        StaticIO.setStreamTokenizerSyntax1(tStreamTokenizer);
        int tokenType = tStreamTokenizer.nextToken();
        ToyModelDataRecord_2 aToyModelDataRecord2;
        String aZoneCode;
        HashMap tLookUpMSOAfromOAHashMap = null;
        CASDataHandler tCASDataHandler = new CASDataHandler();
        if (_Aggregation.equalsIgnoreCase("MSOA")) {
            tLookUpMSOAfromOAHashMap = tCASDataHandler.get_LookUpMSOAfromOAHashMap();
        }
        short _HSARDataRecordAGEH;
        short _HSARDataRecordECONACH;
        short _HSARDataRecordMARSTAH;
        short _ISARDataRecordAGE0;
        short _ISARDataRecordECONACT;
        short _ISARDataRecordMARSTAT;
        boolean _SEX;
        short _HEALTH;
        Counts _Counts;
        TreeMap result = new TreeMap();
        while (tokenType != StreamTokenizer.TT_EOF) {
            switch (tokenType) {
                case StreamTokenizer.TT_WORD:
                    _PeopleWhoseGeneralHealthWasGood = 0;
                    _PeopleWhoseGeneralHealthWasFairlyGood = 0;
                    _PeopleWhoseGeneralHealthWasNotGood = 0;
                    _PeopleWithLimitingLongTermIllness = 0;
                    _LoneParentHouseholdsWithChildren = 0;
                    _OneFamilyAndNoChildren = 0;
                    _MarriedOrCohabitingCoupleWithChildren = 0;
                    _FemalesAge0to4 = 0;
                    _FemalesAge5to9 = 0;
                    _FemalesAge10to14 = 0;
                    _FemalesAge15to19 = 0;
                    _FemalesAge20to24 = 0;
                    _FemalesAge25to29 = 0;
                    _FemalesAge30to34 = 0;
                    _FemalesAge35to39 = 0;
                    _FemalesAge40to44 = 0;
                    _FemalesAge45to49 = 0;
                    _FemalesAge50to54 = 0;
                    _FemalesAge55to59 = 0;
                    _FemalesAge60to64 = 0;
                    _FemalesAge65to69 = 0;
                    _FemalesAge70to74 = 0;
                    _FemalesAge75to79 = 0;
                    _FemalesAge80AndOver = 0;
                    _FemalesMarriedAge0to15 = 0;
                    _FemalesMarriedAge16to19 = 0;
                    _FemalesMarriedAge20to24 = 0;
                    _FemalesMarriedAge25to29 = 0;
                    _FemalesMarriedAge30to34 = 0;
                    _FemalesMarriedAge35to39 = 0;
                    _FemalesMarriedAge40to44 = 0;
                    _FemalesMarriedAge45to49 = 0;
                    _FemalesMarriedAge50to54 = 0;
                    _FemalesMarriedAge55to59 = 0;
                    _FemalesMarriedAge60to64 = 0;
                    _FemalesMarriedAge65to74 = 0;
                    _FemalesMarriedAge75to79 = 0;
                    _FemalesMarriedAge80AndOver = 0;
                    _FemalesAge16to24Unemployed = 0;
                    _FemalesAge16to74 = 0;
                    _FemalesAge16to74EconomicallyActiveEmployed = 0;
                    _FemalesAge16to74EconomicallyActiveUnemployed = 0;
                    _FemalesAge16to74EconomicallyInactive = 0;
                    _FemalesAge50AndOverUnemployed = 0;
                    _MalesAge0to4 = 0;
                    _MalesAge5to9 = 0;
                    _MalesAge10to14 = 0;
                    _MalesAge15to19 = 0;
                    _MalesAge20to24 = 0;
                    _MalesAge25to29 = 0;
                    _MalesAge30to34 = 0;
                    _MalesAge35to39 = 0;
                    _MalesAge40to44 = 0;
                    _MalesAge45to49 = 0;
                    _MalesAge50to54 = 0;
                    _MalesAge55to59 = 0;
                    _MalesAge60to64 = 0;
                    _MalesAge65to69 = 0;
                    _MalesAge70to74 = 0;
                    _MalesAge75to79 = 0;
                    _MalesAge80AndOver = 0;
                    _MalesMarriedAge0to15 = 0;
                    _MalesMarriedAge16to19 = 0;
                    _MalesMarriedAge20to24 = 0;
                    _MalesMarriedAge25to29 = 0;
                    _MalesMarriedAge30to34 = 0;
                    _MalesMarriedAge35to39 = 0;
                    _MalesMarriedAge40to44 = 0;
                    _MalesMarriedAge45to49 = 0;
                    _MalesMarriedAge50to54 = 0;
                    _MalesMarriedAge55to59 = 0;
                    _MalesMarriedAge60to64 = 0;
                    _MalesMarriedAge65to74 = 0;
                    _MalesMarriedAge75to79 = 0;
                    _MalesMarriedAge80AndOver = 0;
                    _MalesAge16to24Unemployed = 0;
                    _MalesAge16to74 = 0;
                    _MalesAge16to74EconomicallyActiveEmployed = 0;
                    _MalesAge16to74EconomicallyActiveUnemployed = 0;
                    _MalesAge16to74EconomicallyInactive = 0;
                    _MalesAge50AndOverUnemployed = 0;
                    aToyModelDataRecord2 = new ToyModelDataRecord_2(
                            tToyModelDataHandler, tStreamTokenizer.sval);
                    if (_Aggregation.equalsIgnoreCase("MSOA")) {
                        aZoneCode = (String) tLookUpMSOAfromOAHashMap.get(new String(aToyModelDataRecord2.getZone_Code()));
                    } else {
                        aZoneCode = String.valueOf(aToyModelDataRecord2.getZone_Code());
                        if (_Aggregation.equalsIgnoreCase("Ward")) {
                            aZoneCode = aZoneCode.substring(0, 6);
                        }
                    }
                    if (aToyModelDataRecord2.tHSARDataRecordID != -9) {
                        _HSARDataRecord = (HSARDataRecord) tHSARDataHandler.getDataRecord(aToyModelDataRecord2.tHSARDataRecordID);
                        _HSARDataRecordAGEH = _HSARDataRecord.get_AGEH();
                        if (_HSARDataRecord.get_HRP()) {
                            short _HSARDataRecordFAMTYPE = _HSARDataRecord.get_FAMTYP();
                            if (_HSARDataRecordFAMTYPE > 0 && _HSARDataRecordFAMTYPE < 3) {
                                _LoneParentHouseholdsWithChildren = 1;
                            }
                            if (_HSARDataRecordFAMTYPE == (short) 3 || _HSARDataRecordFAMTYPE == (short) 6) {
                                _OneFamilyAndNoChildren = 1;
                            }
                            if (_HSARDataRecordFAMTYPE == (short) 4 || _HSARDataRecordFAMTYPE == (short) 5 || _HSARDataRecordFAMTYPE == (short) 7 || _HSARDataRecordFAMTYPE == (short) 8) {
                                _MarriedOrCohabitingCoupleWithChildren = 1;
                            }
                        }
                        _SEX = _HSARDataRecord.get_SEX();
                        _HSARDataRecordECONACH = _HSARDataRecord.get_ECONACH();
                        _HSARDataRecordMARSTAH = _HSARDataRecord.get_MARSTAH();
                        _HEALTH = _HSARDataRecord.get_HEALTH();
                        if (_HSARDataRecord.get_LLTI() == (short) 1) {
                            _PeopleWithLimitingLongTermIllness = 1;
                        }
                        if (_SEX) {
                            // Male
                            if (_HSARDataRecordAGEH < 5) {
                                _MalesAge0to4 = 1;
                            } else {
                                if (_HSARDataRecordAGEH < 10) {
                                    _MalesAge5to9 = 1;
                                } else {
                                    if (_HSARDataRecordAGEH < 15) {
                                        _MalesAge10to14 = 1;
                                    } else {
                                        if (_HSARDataRecordAGEH < 20) {
                                            _MalesAge15to19 = 1;
                                        } else {
                                            if (_HSARDataRecordAGEH < 25) {
                                                _MalesAge20to24 = 1;
                                            } else {
                                                if (_HSARDataRecordAGEH < 30) {
                                                    _MalesAge25to29 = 1;
                                                } else {
                                                    if (_HSARDataRecordAGEH < 35) {
                                                        _MalesAge30to34 = 1;
                                                    } else {
                                                        if (_HSARDataRecordAGEH < 40) {
                                                            _MalesAge35to39 = 1;
                                                        } else {
                                                            if (_HSARDataRecordAGEH < 45) {
                                                                _MalesAge40to44 = 1;
                                                            } else {
                                                                if (_HSARDataRecordAGEH < 50) {
                                                                    _MalesAge45to49 = 1;
                                                                } else {
                                                                    if (_HSARDataRecordAGEH < 55) {
                                                                        _MalesAge50to54 = 1;
                                                                    } else {
                                                                        if (_HSARDataRecordAGEH < 60) {
                                                                            _MalesAge55to59 = 1;
                                                                        } else {
                                                                            if (_HSARDataRecordAGEH < 65) {
                                                                                _MalesAge60to64 = 1;
                                                                            } else {
                                                                                if (_HSARDataRecordAGEH < 70) {
                                                                                    _MalesAge65to69 = 1;
                                                                                } else {
                                                                                    if (_HSARDataRecordAGEH < 75) {
                                                                                        _MalesAge70to74 = 1;
                                                                                    } else {
                                                                                        if (_HSARDataRecordAGEH < 80) {
                                                                                            _MalesAge75to79 = 1;
                                                                                        } else {
                                                                                            _MalesAge80AndOver = 1;
                                                                                        }
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            // Married
                            if (_HSARDataRecordMARSTAH == (short) 2) {
                                if (_HSARDataRecordAGEH < 16) {
                                    _MalesMarriedAge0to15 = 1;
                                } else {
                                    if (_HSARDataRecordAGEH < 20) {
                                        _MalesMarriedAge16to19 = 1;
                                    } else {
                                        if (_HSARDataRecordAGEH < 25) {
                                            _MalesMarriedAge20to24 = 1;
                                        } else {
                                            if (_HSARDataRecordAGEH < 30) {
                                                _MalesMarriedAge25to29 = 1;
                                            } else {
                                                if (_HSARDataRecordAGEH < 35) {
                                                    _MalesMarriedAge30to34 = 1;
                                                } else {
                                                    if (_HSARDataRecordAGEH < 40) {
                                                        _MalesMarriedAge35to39 = 1;
                                                    } else {
                                                        if (_HSARDataRecordAGEH < 45) {
                                                            _MalesMarriedAge40to44 = 1;
                                                        } else {
                                                            if (_HSARDataRecordAGEH < 50) {
                                                                _MalesMarriedAge45to49 = 1;
                                                            } else {
                                                                if (_HSARDataRecordAGEH < 55) {
                                                                    _MalesMarriedAge50to54 = 1;
                                                                } else {
                                                                    if (_HSARDataRecordAGEH < 60) {
                                                                        _MalesMarriedAge55to59 = 1;
                                                                    } else {
                                                                        if (_HSARDataRecordAGEH < 65) {
                                                                            _MalesMarriedAge60to64 = 1;
                                                                        } else {
                                                                            if (_HSARDataRecordAGEH < 70) {
                                                                                _MalesMarriedAge65to74 = 1;
                                                                            } else {
                                                                                if (_HSARDataRecordAGEH < 80) {
                                                                                    _MalesMarriedAge75to79 = 1;
                                                                                } else {
                                                                                    _MalesMarriedAge80AndOver = 1;
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            // Unemployed
                            if (_HSARDataRecordECONACH == (short) 2) {
                                if (_HSARDataRecordAGEH > 15) {
                                    if (_HSARDataRecordAGEH < 25) {
                                        _MalesAge16to24Unemployed = 1;
                                    }
                                    if (_HSARDataRecordAGEH < 75) {
                                        _MalesAge16to74EconomicallyActiveUnemployed = 1;
                                    // if ( _HSARDataRecordAGEH > 49 ) {
                                    // _MalesAge50AndOverUnemployed = 1;
                                    // }
                                    }
                                }
                                if (_HSARDataRecordAGEH > 49) {
                                    _MalesAge50AndOverUnemployed = 1;
                                }
                            } else {
                                if (_HSARDataRecordAGEH > 15 && _HSARDataRecordAGEH < 75) {
                                    _MalesAge16to74 = 1;
                                    if (_HSARDataRecordECONACH == (short) 1) {
                                        _MalesAge16to74EconomicallyActiveEmployed = 1;
                                    }
                                    if (_HSARDataRecordECONACH > 2) {
                                        _MalesAge16to74EconomicallyInactive = 1;
                                    }
                                }
                            }
                        } else {
                            // Female
                            if (_HSARDataRecordAGEH < 5) {
                                _FemalesAge0to4 = 1;
                            } else {
                                if (_HSARDataRecordAGEH < 10) {
                                    _FemalesAge5to9 = 1;
                                } else {
                                    if (_HSARDataRecordAGEH < 15) {
                                        _FemalesAge10to14 = 1;
                                    } else {
                                        if (_HSARDataRecordAGEH < 20) {
                                            _FemalesAge15to19 = 1;
                                        } else {
                                            if (_HSARDataRecordAGEH < 25) {
                                                _FemalesAge20to24 = 1;
                                            } else {
                                                if (_HSARDataRecordAGEH < 30) {
                                                    _FemalesAge25to29 = 1;
                                                } else {
                                                    if (_HSARDataRecordAGEH < 35) {
                                                        _FemalesAge30to34 = 1;
                                                    } else {
                                                        if (_HSARDataRecordAGEH < 40) {
                                                            _FemalesAge35to39 = 1;
                                                        } else {
                                                            if (_HSARDataRecordAGEH < 45) {
                                                                _FemalesAge40to44 = 1;
                                                            } else {
                                                                if (_HSARDataRecordAGEH < 50) {
                                                                    _FemalesAge45to49 = 1;
                                                                } else {
                                                                    if (_HSARDataRecordAGEH < 55) {
                                                                        _FemalesAge50to54 = 1;
                                                                    } else {
                                                                        if (_HSARDataRecordAGEH < 60) {
                                                                            _FemalesAge55to59 = 1;
                                                                        } else {
                                                                            if (_HSARDataRecordAGEH < 65) {
                                                                                _FemalesAge60to64 = 1;
                                                                            } else {
                                                                                if (_HSARDataRecordAGEH < 70) {
                                                                                    _FemalesAge65to69 = 1;
                                                                                } else {
                                                                                    if (_HSARDataRecordAGEH < 75) {
                                                                                        _FemalesAge70to74 = 1;
                                                                                    } else {
                                                                                        if (_HSARDataRecordAGEH < 80) {
                                                                                            _FemalesAge75to79 = 1;
                                                                                        } else {
                                                                                            _FemalesAge80AndOver = 1;
                                                                                        }
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            // Married
                            if (_HSARDataRecordMARSTAH == (short) 2) {
                                if (_HSARDataRecordAGEH < 16) {
                                    _FemalesMarriedAge0to15 = 1;
                                } else {
                                    if (_HSARDataRecordAGEH < 20) {
                                        _FemalesMarriedAge16to19 = 1;
                                    } else {
                                        if (_HSARDataRecordAGEH < 25) {
                                            _FemalesMarriedAge20to24 = 1;
                                        } else {
                                            if (_HSARDataRecordAGEH < 30) {
                                                _FemalesMarriedAge25to29 = 1;
                                            } else {
                                                if (_HSARDataRecordAGEH < 35) {
                                                    _FemalesMarriedAge30to34 = 1;
                                                } else {
                                                    if (_HSARDataRecordAGEH < 40) {
                                                        _FemalesMarriedAge35to39 = 1;
                                                    } else {
                                                        if (_HSARDataRecordAGEH < 45) {
                                                            _FemalesMarriedAge40to44 = 1;
                                                        } else {
                                                            if (_HSARDataRecordAGEH < 50) {
                                                                _FemalesMarriedAge45to49 = 1;
                                                            } else {
                                                                if (_HSARDataRecordAGEH < 55) {
                                                                    _FemalesMarriedAge50to54 = 1;
                                                                } else {
                                                                    if (_HSARDataRecordAGEH < 60) {
                                                                        _FemalesMarriedAge55to59 = 1;
                                                                    } else {
                                                                        if (_HSARDataRecordAGEH < 65) {
                                                                            _FemalesMarriedAge60to64 = 1;
                                                                        } else {
                                                                            if (_HSARDataRecordAGEH < 70) {
                                                                                _FemalesMarriedAge65to74 = 1;
                                                                            } else {
                                                                                if (_HSARDataRecordAGEH < 80) {
                                                                                    _FemalesMarriedAge75to79 = 1;
                                                                                } else {
                                                                                    _FemalesMarriedAge80AndOver = 1;
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            // Unemployed
                            if (_HSARDataRecordECONACH == (short) 2) {
                                if (_HSARDataRecordAGEH > 15) {
                                    if (_HSARDataRecordAGEH < 25) {
                                        _FemalesAge16to24Unemployed = 1;
                                    }
                                    if (_HSARDataRecordAGEH < 75) {
                                        _FemalesAge16to74EconomicallyActiveUnemployed = 1;
                                    // if ( _HSARDataRecordAGEH > 49 ) {
                                    // _FemalesAge50AndOverUnemployed = 1;
                                    // }
                                    }
                                }
                                if (_HSARDataRecordAGEH > 49) {
                                    _FemalesAge50AndOverUnemployed = 1;
                                }
                            } else {
                                if (_HSARDataRecordAGEH > 15 && _HSARDataRecordAGEH < 75) {
                                    _FemalesAge16to74 = 1;
                                    if (_HSARDataRecordECONACH == (short) 1) {
                                        _FemalesAge16to74EconomicallyActiveEmployed = 1;
                                    }
                                    if (_HSARDataRecordECONACH > 2) {
                                        _FemalesAge16to74EconomicallyInactive = 1;
                                    }
                                }
                            }
                        }
                    } else {
                        _ISARDataRecord = (ISARDataRecord) tISARDataHandler.getDataRecord(aToyModelDataRecord2.tISARDataRecordID);
                        _ISARDataRecordAGE0 = _ISARDataRecord.get_AGE0();
                        _SEX = _ISARDataRecord.get_SEX();
                        _ISARDataRecordECONACT = _ISARDataRecord.get_ECONACT();
                        _ISARDataRecordMARSTAT = _ISARDataRecord.get_MARSTAT();
                        _HEALTH = _ISARDataRecord.get_HEALTH();
                        if (_ISARDataRecord.get_LLTI() == (short) 1) {
                            _PeopleWithLimitingLongTermIllness = 1;
                        }
                        if (_SEX) {
                            // Male
                            if (_ISARDataRecordAGE0 < 5) {
                                _MalesAge0to4 = 1;
                            } else {
                                if (_ISARDataRecordAGE0 < 10) {
                                    _MalesAge5to9 = 1;
                                } else {
                                    if (_ISARDataRecordAGE0 < 15) {
                                        _MalesAge10to14 = 1;
                                    } else {
                                        if (_ISARDataRecordAGE0 < 20) {
                                            _MalesAge15to19 = 1;
                                        } else {
                                            if (_ISARDataRecordAGE0 < 25) {
                                                _MalesAge20to24 = 1;
                                            } else {
                                                if (_ISARDataRecordAGE0 < 30) {
                                                    _MalesAge25to29 = 1;
                                                } else {
                                                    if (_ISARDataRecordAGE0 < 35) {
                                                        _MalesAge30to34 = 1;
                                                    } else {
                                                        if (_ISARDataRecordAGE0 < 40) {
                                                            _MalesAge35to39 = 1;
                                                        } else {
                                                            if (_ISARDataRecordAGE0 < 45) {
                                                                _MalesAge40to44 = 1;
                                                            } else {
                                                                if (_ISARDataRecordAGE0 < 50) {
                                                                    _MalesAge45to49 = 1;
                                                                } else {
                                                                    if (_ISARDataRecordAGE0 < 55) {
                                                                        _MalesAge50to54 = 1;
                                                                    } else {
                                                                        if (_ISARDataRecordAGE0 < 60) {
                                                                            _MalesAge55to59 = 1;
                                                                        } else {
                                                                            if (_ISARDataRecordAGE0 < 65) {
                                                                                _MalesAge60to64 = 1;
                                                                            } else {
                                                                                if (_ISARDataRecordAGE0 < 70) {
                                                                                    _MalesAge65to69 = 1;
                                                                                } else {
                                                                                    if (_ISARDataRecordAGE0 < 75) {
                                                                                        _MalesAge70to74 = 1;
                                                                                    } else {
                                                                                        if (_ISARDataRecordAGE0 < 80) {
                                                                                            _MalesAge75to79 = 1;
                                                                                        } else {
                                                                                            _MalesAge80AndOver = 1;
                                                                                        }
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            // Married
                            if (_ISARDataRecordMARSTAT == (short) 2 || _ISARDataRecordMARSTAT == (short) 3) {
                                if (_ISARDataRecordAGE0 < 16) {
                                    _MalesMarriedAge0to15 = 1;
                                } else {
                                    if (_ISARDataRecordAGE0 < 20) {
                                        _MalesMarriedAge16to19 = 1;
                                    } else {
                                        if (_ISARDataRecordAGE0 < 25) {
                                            _MalesMarriedAge20to24 = 1;
                                        } else {
                                            if (_ISARDataRecordAGE0 < 30) {
                                                _MalesMarriedAge25to29 = 1;
                                            } else {
                                                if (_ISARDataRecordAGE0 < 35) {
                                                    _MalesMarriedAge30to34 = 1;
                                                } else {
                                                    if (_ISARDataRecordAGE0 < 40) {
                                                        _MalesMarriedAge35to39 = 1;
                                                    } else {
                                                        if (_ISARDataRecordAGE0 < 45) {
                                                            _MalesMarriedAge40to44 = 1;
                                                        } else {
                                                            if (_ISARDataRecordAGE0 < 50) {
                                                                _MalesMarriedAge45to49 = 1;
                                                            } else {
                                                                if (_ISARDataRecordAGE0 < 55) {
                                                                    _MalesMarriedAge50to54 = 1;
                                                                } else {
                                                                    if (_ISARDataRecordAGE0 < 60) {
                                                                        _MalesMarriedAge55to59 = 1;
                                                                    } else {
                                                                        if (_ISARDataRecordAGE0 < 65) {
                                                                            _MalesMarriedAge60to64 = 1;
                                                                        } else {
                                                                            if (_ISARDataRecordAGE0 < 70) {
                                                                                _MalesMarriedAge65to74 = 1;
                                                                            } else {
                                                                                if (_ISARDataRecordAGE0 < 80) {
                                                                                    _MalesMarriedAge75to79 = 1;
                                                                                } else {
                                                                                    _MalesMarriedAge80AndOver = 1;
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            // Unemployed
                            if (_ISARDataRecordECONACT > 5 && _ISARDataRecordECONACT < 9) {
                                if (_ISARDataRecordAGE0 > 15) {
                                    if (_ISARDataRecordAGE0 < 25) {
                                        _MalesAge16to24Unemployed = 1;
                                    }
                                    if (_ISARDataRecordAGE0 < 75) {
                                        _MalesAge16to74EconomicallyActiveUnemployed = 1;
                                    // if ( _ISARDataRecordAGE0 > 49 ) {
                                    // _MalesAge50AndOverUnemployed = 1;
                                    // }
                                    }
                                }
                                if (_ISARDataRecordAGE0 > 49) {
                                    _MalesAge50AndOverUnemployed = 1;
                                }
                            } else {
                                if (_ISARDataRecordAGE0 > 15 && _ISARDataRecordAGE0 < 75) {
                                    if (_ISARDataRecordECONACT > 0 && _ISARDataRecordECONACT < 7) {
                                        _MalesAge16to74EconomicallyActiveEmployed = 1;
                                    }
                                    if (_ISARDataRecordECONACT > 8) {
                                        _MalesAge16to74EconomicallyInactive = 1;
                                    }
                                }
                            }
                        } else {
                            // Female
                            if (_ISARDataRecordAGE0 < 5) {
                                _FemalesAge0to4 = 1;
                            } else {
                                if (_ISARDataRecordAGE0 < 10) {
                                    _FemalesAge5to9 = 1;
                                } else {
                                    if (_ISARDataRecordAGE0 < 15) {
                                        _FemalesAge10to14 = 1;
                                    } else {
                                        if (_ISARDataRecordAGE0 < 20) {
                                            _FemalesAge15to19 = 1;
                                        } else {
                                            if (_ISARDataRecordAGE0 < 25) {
                                                _FemalesAge20to24 = 1;
                                            } else {
                                                if (_ISARDataRecordAGE0 < 30) {
                                                    _FemalesAge25to29 = 1;
                                                } else {
                                                    if (_ISARDataRecordAGE0 < 35) {
                                                        _FemalesAge30to34 = 1;
                                                    } else {
                                                        if (_ISARDataRecordAGE0 < 40) {
                                                            _FemalesAge35to39 = 1;
                                                        } else {
                                                            if (_ISARDataRecordAGE0 < 45) {
                                                                _FemalesAge40to44 = 1;
                                                            } else {
                                                                if (_ISARDataRecordAGE0 < 50) {
                                                                    _FemalesAge45to49 = 1;
                                                                } else {
                                                                    if (_ISARDataRecordAGE0 < 55) {
                                                                        _FemalesAge50to54 = 1;
                                                                    } else {
                                                                        if (_ISARDataRecordAGE0 < 60) {
                                                                            _FemalesAge55to59 = 1;
                                                                        } else {
                                                                            if (_ISARDataRecordAGE0 < 65) {
                                                                                _FemalesAge60to64 = 1;
                                                                            } else {
                                                                                if (_ISARDataRecordAGE0 < 70) {
                                                                                    _FemalesAge65to69 = 1;
                                                                                } else {
                                                                                    if (_ISARDataRecordAGE0 < 75) {
                                                                                        _FemalesAge70to74 = 1;
                                                                                    } else {
                                                                                        if (_ISARDataRecordAGE0 < 80) {
                                                                                            _FemalesAge75to79 = 1;
                                                                                        } else {
                                                                                            _FemalesAge80AndOver = 1;
                                                                                        }
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            // Married
                            if (_ISARDataRecordMARSTAT == (short) 2 || _ISARDataRecordMARSTAT == (short) 3) {
                                if (_ISARDataRecordAGE0 < 16) {
                                    _FemalesMarriedAge0to15 = 1;
                                } else {
                                    if (_ISARDataRecordAGE0 < 20) {
                                        _FemalesMarriedAge16to19 = 1;
                                    } else {
                                        if (_ISARDataRecordAGE0 < 25) {
                                            _FemalesMarriedAge20to24 = 1;
                                        } else {
                                            if (_ISARDataRecordAGE0 < 30) {
                                                _FemalesMarriedAge25to29 = 1;
                                            } else {
                                                if (_ISARDataRecordAGE0 < 35) {
                                                    _FemalesMarriedAge30to34 = 1;
                                                } else {
                                                    if (_ISARDataRecordAGE0 < 40) {
                                                        _FemalesMarriedAge35to39 = 1;
                                                    } else {
                                                        if (_ISARDataRecordAGE0 < 45) {
                                                            _FemalesMarriedAge40to44 = 1;
                                                        } else {
                                                            if (_ISARDataRecordAGE0 < 50) {
                                                                _FemalesMarriedAge45to49 = 1;
                                                            } else {
                                                                if (_ISARDataRecordAGE0 < 55) {
                                                                    _FemalesMarriedAge50to54 = 1;
                                                                } else {
                                                                    if (_ISARDataRecordAGE0 < 60) {
                                                                        _FemalesMarriedAge55to59 = 1;
                                                                    } else {
                                                                        if (_ISARDataRecordAGE0 < 65) {
                                                                            _FemalesMarriedAge60to64 = 1;
                                                                        } else {
                                                                            if (_ISARDataRecordAGE0 < 70) {
                                                                                _FemalesMarriedAge65to74 = 1;
                                                                            } else {
                                                                                if (_ISARDataRecordAGE0 < 80) {
                                                                                    _FemalesMarriedAge75to79 = 1;
                                                                                } else {
                                                                                    _FemalesMarriedAge80AndOver = 1;
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                            // Unemployed
                            if (_ISARDataRecordECONACT > 5 && _ISARDataRecordECONACT < 9) {
                                if (_ISARDataRecordAGE0 > 15) {
                                    if (_ISARDataRecordAGE0 < 25) {
                                        _FemalesAge16to24Unemployed = 1;
                                    }
                                    if (_ISARDataRecordAGE0 < 75) {
                                        _FemalesAge16to74EconomicallyActiveUnemployed = 1;
                                    // if ( _ISARDataRecordAGE0 > 49 ) {
                                    // _FemalesAge50AndOverUnemployed = 1;
                                    // }
                                    }
                                }
                                if (_ISARDataRecordAGE0 > 49) {
                                    _FemalesAge50AndOverUnemployed = 1;
                                }
                            } else {
                                if (_ISARDataRecordAGE0 > 15 && _ISARDataRecordAGE0 < 75) {
                                    if (_ISARDataRecordECONACT > 0 && _ISARDataRecordECONACT < 7) {
                                        _FemalesAge16to74EconomicallyActiveEmployed = 1;
                                    }
                                    if (_ISARDataRecordECONACT > 8) {
                                        _FemalesAge16to74EconomicallyInactive = 1;
                                    }
                                }
                            }
                        }
                    }
                    if (_HEALTH == (short) 1) {
                        _PeopleWhoseGeneralHealthWasGood = 1;
                    }
                    if (_HEALTH == (short) 2) {
                        _PeopleWhoseGeneralHealthWasFairlyGood = 1;
                    }
                    if (_HEALTH == (short) 3) {
                        _PeopleWhoseGeneralHealthWasNotGood = 1;
                    }
                    if (result.containsKey(aZoneCode)) {
                        _Counts = (Counts) result.get(aZoneCode);
                        result.remove(aZoneCode);
                        _Counts.addToCounts(_PeopleWhoseGeneralHealthWasGood,
                                _PeopleWhoseGeneralHealthWasFairlyGood,
                                _PeopleWhoseGeneralHealthWasNotGood,
                                _PeopleWithLimitingLongTermIllness,
                                _LoneParentHouseholdsWithChildren,
                                _OneFamilyAndNoChildren,
                                _MarriedOrCohabitingCoupleWithChildren,
                                _FemalesAge0to4, _FemalesAge5to9,
                                _FemalesAge10to14, _FemalesAge15to19,
                                _FemalesAge20to24, _FemalesAge25to29,
                                _FemalesAge30to34, _FemalesAge35to39,
                                _FemalesAge40to44, _FemalesAge45to49,
                                _FemalesAge50to54, _FemalesAge55to59,
                                _FemalesAge60to64, _FemalesAge65to69,
                                _FemalesAge70to74, _FemalesAge75to79,
                                _FemalesAge80AndOver, _FemalesMarriedAge0to15,
                                _FemalesMarriedAge16to19, _FemalesMarriedAge20to24,
                                _FemalesMarriedAge25to29, _FemalesMarriedAge30to34,
                                _FemalesMarriedAge35to39, _FemalesMarriedAge40to44,
                                _FemalesMarriedAge45to49, _FemalesMarriedAge50to54,
                                _FemalesMarriedAge55to59, _FemalesMarriedAge60to64,
                                _FemalesMarriedAge65to74, _FemalesMarriedAge75to79,
                                _FemalesMarriedAge80AndOver,
                                _FemalesAge16to24Unemployed, _FemalesAge16to74,
                                _FemalesAge16to74EconomicallyActiveEmployed,
                                _FemalesAge16to74EconomicallyActiveUnemployed,
                                _FemalesAge16to74EconomicallyInactive,
                                _FemalesAge50AndOverUnemployed, _MalesAge0to4,
                                _MalesAge5to9, _MalesAge10to14, _MalesAge15to19,
                                _MalesAge20to24, _MalesAge25to29, _MalesAge30to34,
                                _MalesAge35to39, _MalesAge40to44, _MalesAge45to49,
                                _MalesAge50to54, _MalesAge55to59, _MalesAge60to64,
                                _MalesAge65to69, _MalesAge70to74, _MalesAge75to79,
                                _MalesAge80AndOver, _MalesMarriedAge0to15,
                                _MalesMarriedAge16to19, _MalesMarriedAge20to24,
                                _MalesMarriedAge25to29, _MalesMarriedAge30to34,
                                _MalesMarriedAge35to39, _MalesMarriedAge40to44,
                                _MalesMarriedAge45to49, _MalesMarriedAge50to54,
                                _MalesMarriedAge55to59, _MalesMarriedAge60to64,
                                _MalesMarriedAge65to74, _MalesMarriedAge75to79,
                                _MalesMarriedAge80AndOver,
                                _MalesAge16to24Unemployed, _MalesAge16to74,
                                _MalesAge16to74EconomicallyActiveEmployed,
                                _MalesAge16to74EconomicallyActiveUnemployed,
                                _MalesAge16to74EconomicallyInactive,
                                _MalesAge50AndOverUnemployed);
                        result.put(aZoneCode, _Counts);
                    } else {
                        _Counts = new Counts();
                        _Counts.addToCounts(_PeopleWhoseGeneralHealthWasGood,
                                _PeopleWhoseGeneralHealthWasFairlyGood,
                                _PeopleWhoseGeneralHealthWasNotGood,
                                _PeopleWithLimitingLongTermIllness,
                                _LoneParentHouseholdsWithChildren,
                                _OneFamilyAndNoChildren,
                                _MarriedOrCohabitingCoupleWithChildren,
                                _FemalesAge0to4, _FemalesAge5to9,
                                _FemalesAge10to14, _FemalesAge15to19,
                                _FemalesAge20to24, _FemalesAge25to29,
                                _FemalesAge30to34, _FemalesAge35to39,
                                _FemalesAge40to44, _FemalesAge45to49,
                                _FemalesAge50to54, _FemalesAge55to59,
                                _FemalesAge60to64, _FemalesAge65to69,
                                _FemalesAge70to74, _FemalesAge75to79,
                                _FemalesAge80AndOver, _FemalesMarriedAge0to15,
                                _FemalesMarriedAge16to19, _FemalesMarriedAge20to24,
                                _FemalesMarriedAge25to29, _FemalesMarriedAge30to34,
                                _FemalesMarriedAge35to39, _FemalesMarriedAge40to44,
                                _FemalesMarriedAge45to49, _FemalesMarriedAge50to54,
                                _FemalesMarriedAge55to59, _FemalesMarriedAge60to64,
                                _FemalesMarriedAge65to74, _FemalesMarriedAge75to79,
                                _FemalesMarriedAge80AndOver,
                                _FemalesAge16to24Unemployed, _FemalesAge16to74,
                                _FemalesAge16to74EconomicallyActiveEmployed,
                                _FemalesAge16to74EconomicallyActiveUnemployed,
                                _FemalesAge16to74EconomicallyInactive,
                                _FemalesAge50AndOverUnemployed, _MalesAge0to4,
                                _MalesAge5to9, _MalesAge10to14, _MalesAge15to19,
                                _MalesAge20to24, _MalesAge25to29, _MalesAge30to34,
                                _MalesAge35to39, _MalesAge40to44, _MalesAge45to49,
                                _MalesAge50to54, _MalesAge55to59, _MalesAge60to64,
                                _MalesAge65to69, _MalesAge70to74, _MalesAge75to79,
                                _MalesAge80AndOver, _MalesMarriedAge0to15,
                                _MalesMarriedAge16to19, _MalesMarriedAge20to24,
                                _MalesMarriedAge25to29, _MalesMarriedAge30to34,
                                _MalesMarriedAge35to39, _MalesMarriedAge40to44,
                                _MalesMarriedAge45to49, _MalesMarriedAge50to54,
                                _MalesMarriedAge55to59, _MalesMarriedAge60to64,
                                _MalesMarriedAge65to74, _MalesMarriedAge75to79,
                                _MalesMarriedAge80AndOver,
                                _MalesAge16to24Unemployed, _MalesAge16to74,
                                _MalesAge16to74EconomicallyActiveEmployed,
                                _MalesAge16to74EconomicallyActiveUnemployed,
                                _MalesAge16to74EconomicallyInactive,
                                _MalesAge50AndOverUnemployed);
                        result.put(aZoneCode, _Counts);
                    }
            }
            tokenType = tStreamTokenizer.nextToken();
        }
        Iterator aIterator = result.keySet().iterator();
        Object key;
        while (aIterator.hasNext()) {
            key = aIterator.next();
            _Counts = (Counts) result.get(key);
            aZoneCode = (String) key;
            write(_Counts, aZoneCode);
        }
        this._FileOutputStream.close();
    }

    protected void write(Counts _Counts, String _ZoneCode) throws IOException {
        String record = new String(_ZoneCode + "," + _Counts._PeopleWhoseGeneralHealthWasGood + "," + _Counts._PeopleWhoseGeneralHealthWasFairlyGood + "," + _Counts._PeopleWhoseGeneralHealthWasNotGood + "," + _Counts._PeopleWithLimitingLongTermIllness + "," + _Counts._LoneParentHouseholdsWithChildren + "," + _Counts._OneFamilyAndNoChildren + "," + _Counts._MarriedOrCohabitingCoupleWithChildren + "," + _Counts._FemalesAge0to4 + "," + _Counts._FemalesAge5to9 + "," + _Counts._FemalesAge10to14 + "," + _Counts._FemalesAge15to19 + "," + _Counts._FemalesAge20to24 + "," + _Counts._FemalesAge25to29 + "," + _Counts._FemalesAge30to34 + "," + _Counts._FemalesAge35to39 + "," + _Counts._FemalesAge40to44 + "," + _Counts._FemalesAge45to49 + "," + _Counts._FemalesAge50to54 + "," + _Counts._FemalesAge55to59 + "," + _Counts._FemalesAge60to64 + "," + _Counts._FemalesAge65to69 + "," + _Counts._FemalesAge70to74 + "," + _Counts._FemalesAge75to79 + "," + _Counts._FemalesAge80AndOver + "," + _Counts._FemalesMarriedAge0to15 + "," + _Counts._FemalesMarriedAge16to19 + "," + _Counts._FemalesMarriedAge20to24 + "," + _Counts._FemalesMarriedAge25to29 + "," + _Counts._FemalesMarriedAge30to34 + "," + _Counts._FemalesMarriedAge35to39 + "," + _Counts._FemalesMarriedAge40to44 + "," + _Counts._FemalesMarriedAge45to49 + "," + _Counts._FemalesMarriedAge50to54 + "," + _Counts._FemalesMarriedAge55to59 + "," + _Counts._FemalesMarriedAge60to64 + "," + _Counts._FemalesMarriedAge65to74 + "," + _Counts._FemalesMarriedAge75to79 + "," + _Counts._FemalesMarriedAge80AndOver + "," + _Counts._FemalesAge16to24Unemployed + "," + _Counts._FemalesAge16to74 + "," + _Counts._FemalesAge16to74EconomicallyActiveEmployed + "," + _Counts._FemalesAge16to74EconomicallyActiveUnemployed + "," + _Counts._FemalesAge16to74EconomicallyInactive + "," + _Counts._FemalesAge50AndOverUnemployed + "," + _Counts._MalesAge0to4 + "," + _Counts._MalesAge5to9 + "," + _Counts._MalesAge10to14 + "," + _Counts._MalesAge15to19 + "," + _Counts._MalesAge20to24 + "," + _Counts._MalesAge25to29 + "," + _Counts._MalesAge30to34 + "," + _Counts._MalesAge35to39 + "," + _Counts._MalesAge40to44 + "," + _Counts._MalesAge45to49 + "," + _Counts._MalesAge50to54 + "," + _Counts._MalesAge55to59 + "," + _Counts._MalesAge60to64 + "," + _Counts._MalesAge65to69 + "," + _Counts._MalesAge70to74 + "," + _Counts._MalesAge75to79 + "," + _Counts._MalesAge80AndOver + "," + _Counts._MalesMarriedAge0to15 + "," + _Counts._MalesMarriedAge16to19 + "," + _Counts._MalesMarriedAge20to24 + "," + _Counts._MalesMarriedAge25to29 + "," + _Counts._MalesMarriedAge30to34 + "," + _Counts._MalesMarriedAge35to39 + "," + _Counts._MalesMarriedAge40to44 + "," + _Counts._MalesMarriedAge45to49 + "," + _Counts._MalesMarriedAge50to54 + "," + _Counts._MalesMarriedAge55to59 + "," + _Counts._MalesMarriedAge60to64 + "," + _Counts._MalesMarriedAge65to74 + "," + _Counts._MalesMarriedAge75to79 + "," + _Counts._MalesMarriedAge80AndOver + "," + _Counts._MalesAge16to24Unemployed + "," + _Counts._MalesAge16to74 + "," + _Counts._MalesAge16to74EconomicallyActiveEmployed + "," + _Counts._MalesAge16to74EconomicallyActiveUnemployed + "," + _Counts._MalesAge16to74EconomicallyInactive + "," + _Counts._MalesAge50AndOverUnemployed);
        System.out.println(record);
        this._FileOutputStream.write(record.getBytes());
        this._FileOutputStream.write(StreamTokenizer.TT_EOL);
        this._FileOutputStream.flush();
    }

    public class Counts {

        int _PeopleWhoseGeneralHealthWasGood;
        int _PeopleWhoseGeneralHealthWasFairlyGood;
        int _PeopleWhoseGeneralHealthWasNotGood;
        int _PeopleWithLimitingLongTermIllness;
        int _LoneParentHouseholdsWithChildren;
        int _OneFamilyAndNoChildren;
        int _MarriedOrCohabitingCoupleWithChildren;
        int _FemalesAge0to4;
        int _FemalesAge5to9;
        int _FemalesAge10to14;
        int _FemalesAge15to19;
        int _FemalesAge20to24;
        int _FemalesAge25to29;
        int _FemalesAge30to34;
        int _FemalesAge35to39;
        int _FemalesAge40to44;
        int _FemalesAge45to49;
        int _FemalesAge50to54;
        int _FemalesAge55to59;
        int _FemalesAge60to64;
        int _FemalesAge65to69;
        int _FemalesAge70to74;
        int _FemalesAge75to79;
        int _FemalesAge80AndOver;
        int _FemalesMarriedAge0to15;
        int _FemalesMarriedAge16to19;
        int _FemalesMarriedAge20to24;
        int _FemalesMarriedAge25to29;
        int _FemalesMarriedAge30to34;
        int _FemalesMarriedAge35to39;
        int _FemalesMarriedAge40to44;
        int _FemalesMarriedAge45to49;
        int _FemalesMarriedAge50to54;
        int _FemalesMarriedAge55to59;
        int _FemalesMarriedAge60to64;
        int _FemalesMarriedAge65to74;
        int _FemalesMarriedAge75to79;
        int _FemalesMarriedAge80AndOver;
        int _FemalesAge16to24Unemployed;
        int _FemalesAge16to74;
        int _FemalesAge16to74EconomicallyActiveEmployed;
        int _FemalesAge16to74EconomicallyActiveUnemployed;
        int _FemalesAge16to74EconomicallyInactive;
        int _FemalesAge50AndOverUnemployed;
        int _MalesAge0to4;
        int _MalesAge5to9;
        int _MalesAge10to14;
        int _MalesAge15to19;
        int _MalesAge20to24;
        int _MalesAge25to29;
        int _MalesAge30to34;
        int _MalesAge35to39;
        int _MalesAge40to44;
        int _MalesAge45to49;
        int _MalesAge50to54;
        int _MalesAge55to59;
        int _MalesAge60to64;
        int _MalesAge65to69;
        int _MalesAge70to74;
        int _MalesAge75to79;
        int _MalesAge80AndOver;
        int _MalesMarriedAge0to15;
        int _MalesMarriedAge16to19;
        int _MalesMarriedAge20to24;
        int _MalesMarriedAge25to29;
        int _MalesMarriedAge30to34;
        int _MalesMarriedAge35to39;
        int _MalesMarriedAge40to44;
        int _MalesMarriedAge45to49;
        int _MalesMarriedAge50to54;
        int _MalesMarriedAge55to59;
        int _MalesMarriedAge60to64;
        int _MalesMarriedAge65to74;
        int _MalesMarriedAge75to79;
        int _MalesMarriedAge80AndOver;
        int _MalesAge16to24Unemployed;
        int _MalesAge16to74;
        int _MalesAge16to74EconomicallyActiveEmployed;
        int _MalesAge16to74EconomicallyActiveUnemployed;
        int _MalesAge16to74EconomicallyInactive;
        int _MalesAge50AndOverUnemployed;

        public Counts() {
            _PeopleWhoseGeneralHealthWasGood = 0;
            _PeopleWhoseGeneralHealthWasFairlyGood = 0;
            _PeopleWhoseGeneralHealthWasNotGood = 0;
            _PeopleWithLimitingLongTermIllness = 0;
            _LoneParentHouseholdsWithChildren = 0;
            _OneFamilyAndNoChildren = 0;
            _MarriedOrCohabitingCoupleWithChildren = 0;
            _FemalesAge0to4 = 0;
            _FemalesAge5to9 = 0;
            _FemalesAge10to14 = 0;
            _FemalesAge15to19 = 0;
            _FemalesAge20to24 = 0;
            _FemalesAge25to29 = 0;
            _FemalesAge30to34 = 0;
            _FemalesAge35to39 = 0;
            _FemalesAge40to44 = 0;
            _FemalesAge45to49 = 0;
            _FemalesAge50to54 = 0;
            _FemalesAge55to59 = 0;
            _FemalesAge60to64 = 0;
            _FemalesAge65to69 = 0;
            _FemalesAge70to74 = 0;
            _FemalesAge75to79 = 0;
            _FemalesAge80AndOver = 0;
            _FemalesMarriedAge0to15 = 0;
            _FemalesMarriedAge16to19 = 0;
            _FemalesMarriedAge20to24 = 0;
            _FemalesMarriedAge25to29 = 0;
            _FemalesMarriedAge30to34 = 0;
            _FemalesMarriedAge35to39 = 0;
            _FemalesMarriedAge40to44 = 0;
            _FemalesMarriedAge45to49 = 0;
            _FemalesMarriedAge50to54 = 0;
            _FemalesMarriedAge55to59 = 0;
            _FemalesMarriedAge60to64 = 0;
            _FemalesMarriedAge65to74 = 0;
            _FemalesMarriedAge75to79 = 0;
            _FemalesMarriedAge80AndOver = 0;
            _FemalesAge16to24Unemployed = 0;
            _FemalesAge16to74 = 0;
            _FemalesAge16to74EconomicallyActiveEmployed = 0;
            _FemalesAge16to74EconomicallyActiveUnemployed = 0;
            _FemalesAge16to74EconomicallyInactive = 0;
            _FemalesAge50AndOverUnemployed = 0;
            _MalesAge0to4 = 0;
            _MalesAge5to9 = 0;
            _MalesAge10to14 = 0;
            _MalesAge15to19 = 0;
            _MalesAge20to24 = 0;
            _MalesAge25to29 = 0;
            _MalesAge30to34 = 0;
            _MalesAge35to39 = 0;
            _MalesAge40to44 = 0;
            _MalesAge45to49 = 0;
            _MalesAge50to54 = 0;
            _MalesAge55to59 = 0;
            _MalesAge60to64 = 0;
            _MalesAge65to69 = 0;
            _MalesAge70to74 = 0;
            _MalesAge75to79 = 0;
            _MalesAge80AndOver = 0;
            _MalesMarriedAge0to15 = 0;
            _MalesMarriedAge16to19 = 0;
            _MalesMarriedAge20to24 = 0;
            _MalesMarriedAge25to29 = 0;
            _MalesMarriedAge30to34 = 0;
            _MalesMarriedAge35to39 = 0;
            _MalesMarriedAge40to44 = 0;
            _MalesMarriedAge45to49 = 0;
            _MalesMarriedAge50to54 = 0;
            _MalesMarriedAge55to59 = 0;
            _MalesMarriedAge60to64 = 0;
            _MalesMarriedAge65to74 = 0;
            _MalesMarriedAge75to79 = 0;
            _MalesMarriedAge80AndOver = 0;
            _MalesAge16to24Unemployed = 0;
            _MalesAge16to74 = 0;
            _MalesAge16to74EconomicallyActiveEmployed = 0;
            _MalesAge16to74EconomicallyActiveUnemployed = 0;
            _MalesAge16to74EconomicallyInactive = 0;
            _MalesAge50AndOverUnemployed = 0;
        }

        public void addToCounts(int _PeopleWhoseGeneralHealthWasGood,
                int _PeopleWhoseGeneralHealthWasFairlyGood,
                int _PeopleWhoseGeneralHealthWasNotGood,
                int _PeopleWithLimitingLongTermIllness,
                int _LoneParentHouseholdsWithChildren,
                int _OneFamilyAndNoChildren,
                int _MarriedOrCohabitingCoupleWithChildren,
                int _FemalesAge0to4, int _FemalesAge5to9,
                int _FemalesAge10to14, int _FemalesAge15to19,
                int _FemalesAge20to24, int _FemalesAge25to29,
                int _FemalesAge30to34, int _FemalesAge35to39,
                int _FemalesAge40to44, int _FemalesAge45to49,
                int _FemalesAge50to54, int _FemalesAge55to59,
                int _FemalesAge60to64, int _FemalesAge65to69,
                int _FemalesAge70to74, int _FemalesAge75to79,
                int _FemalesAge80AndOver, int _FemalesMarriedAge0to15,
                int _FemalesMarriedAge16to19, int _FemalesMarriedAge20to24,
                int _FemalesMarriedAge25to29, int _FemalesMarriedAge30to34,
                int _FemalesMarriedAge35to39, int _FemalesMarriedAge40to44,
                int _FemalesMarriedAge45to49, int _FemalesMarriedAge50to54,
                int _FemalesMarriedAge55to59, int _FemalesMarriedAge60to64,
                int _FemalesMarriedAge65to74, int _FemalesMarriedAge75to79,
                int _FemalesMarriedAge80AndOver,
                int _FemalesAge16to24Unemployed, int _FemalesAge16to74,
                int _FemalesAge16to74EconomicallyActiveEmployed,
                int _FemalesAge16to74EconomicallyActiveUnemployed,
                int _FemalesAge16to74EconomicallyInactive,
                int _FemalesAge50AndOverUnemployed, int _MalesAge0to4,
                int _MalesAge5to9, int _MalesAge10to14, int _MalesAge15to19,
                int _MalesAge20to24, int _MalesAge25to29, int _MalesAge30to34,
                int _MalesAge35to39, int _MalesAge40to44, int _MalesAge45to49,
                int _MalesAge50to54, int _MalesAge55to59, int _MalesAge60to64,
                int _MalesAge65to69, int _MalesAge70to74, int _MalesAge75to79,
                int _MalesAge80AndOver, int _MalesMarriedAge0to15,
                int _MalesMarriedAge16to19, int _MalesMarriedAge20to24,
                int _MalesMarriedAge25to29, int _MalesMarriedAge30to34,
                int _MalesMarriedAge35to39, int _MalesMarriedAge40to44,
                int _MalesMarriedAge45to49, int _MalesMarriedAge50to54,
                int _MalesMarriedAge55to59, int _MalesMarriedAge60to64,
                int _MalesMarriedAge65to74, int _MalesMarriedAge75to79,
                int _MalesMarriedAge80AndOver, int _MalesAge16to24Unemployed,
                int _MalesAge16to74,
                int _MalesAge16to74EconomicallyActiveEmployed,
                int _MalesAge16to74EconomicallyActiveUnemployed,
                int _MalesAge16to74EconomicallyInactive,
                int _MalesAge50AndOverUnemployed) {
            this._PeopleWhoseGeneralHealthWasGood += _PeopleWhoseGeneralHealthWasGood;
            this._PeopleWhoseGeneralHealthWasFairlyGood += _PeopleWhoseGeneralHealthWasFairlyGood;
            this._PeopleWhoseGeneralHealthWasNotGood += _PeopleWhoseGeneralHealthWasNotGood;
            this._PeopleWithLimitingLongTermIllness += _PeopleWithLimitingLongTermIllness;
            // Household Composition:
            this._LoneParentHouseholdsWithChildren += _LoneParentHouseholdsWithChildren;
            this._OneFamilyAndNoChildren += _OneFamilyAndNoChildren;
            this._MarriedOrCohabitingCoupleWithChildren += _MarriedOrCohabitingCoupleWithChildren;
            // Female:
            // Age:
            this._FemalesAge0to4 += _FemalesAge0to4;
            this._FemalesAge5to9 += _FemalesAge5to9;
            this._FemalesAge10to14 += _FemalesAge10to14;
            this._FemalesAge15to19 += _FemalesAge15to19;
            this._FemalesAge20to24 += _FemalesAge20to24;
            this._FemalesAge25to29 += _FemalesAge25to29;
            this._FemalesAge30to34 += _FemalesAge30to34;
            this._FemalesAge35to39 += _FemalesAge35to39;
            this._FemalesAge40to44 += _FemalesAge40to44;
            this._FemalesAge45to49 += _FemalesAge45to49;
            this._FemalesAge50to54 += _FemalesAge50to54;
            this._FemalesAge55to59 += _FemalesAge55to59;
            this._FemalesAge60to64 += _FemalesAge60to64;
            this._FemalesAge65to69 += _FemalesAge65to69;
            this._FemalesAge70to74 += _FemalesAge70to74;
            this._FemalesAge75to79 += _FemalesAge75to79;
            this._FemalesAge80AndOver += _FemalesAge80AndOver;
            // Marriage Age:
            this._FemalesMarriedAge0to15 += _FemalesMarriedAge0to15;
            this._FemalesMarriedAge16to19 += _FemalesMarriedAge16to19;
            this._FemalesMarriedAge20to24 += _FemalesMarriedAge20to24;
            this._FemalesMarriedAge25to29 += _FemalesMarriedAge25to29;
            this._FemalesMarriedAge30to34 += _FemalesMarriedAge30to34;
            this._FemalesMarriedAge35to39 += _FemalesMarriedAge35to39;
            this._FemalesMarriedAge40to44 += _FemalesMarriedAge40to44;
            this._FemalesMarriedAge45to49 += _FemalesMarriedAge45to49;
            this._FemalesMarriedAge50to54 += _FemalesMarriedAge50to54;
            this._FemalesMarriedAge55to59 += _FemalesMarriedAge55to59;
            this._FemalesMarriedAge60to64 += _FemalesMarriedAge60to64;
            this._FemalesMarriedAge65to74 += _FemalesMarriedAge65to74;
            this._FemalesMarriedAge75to79 += _FemalesMarriedAge75to79;
            this._FemalesMarriedAge80AndOver += _FemalesMarriedAge80AndOver;
            // Economic Activity:
            this._FemalesAge16to24Unemployed += _FemalesAge16to24Unemployed;
            this._FemalesAge16to74 += _FemalesAge16to74;
            this._FemalesAge16to74EconomicallyActiveEmployed += _FemalesAge16to74EconomicallyActiveEmployed;
            this._FemalesAge16to74EconomicallyActiveUnemployed += _FemalesAge16to74EconomicallyActiveUnemployed;
            this._FemalesAge16to74EconomicallyInactive += _FemalesAge16to74EconomicallyInactive;
            this._FemalesAge50AndOverUnemployed += _FemalesAge50AndOverUnemployed;

            // Male:
            // Age:
            this._MalesAge0to4 += _MalesAge0to4;
            this._MalesAge5to9 += _MalesAge5to9;
            this._MalesAge10to14 += _MalesAge10to14;
            this._MalesAge15to19 += _MalesAge15to19;
            this._MalesAge20to24 += _MalesAge20to24;
            this._MalesAge25to29 += _MalesAge25to29;
            this._MalesAge30to34 += _MalesAge30to34;
            this._MalesAge35to39 += _MalesAge35to39;
            this._MalesAge40to44 += _MalesAge40to44;
            this._MalesAge45to49 += _MalesAge45to49;
            this._MalesAge50to54 += _MalesAge50to54;
            this._MalesAge55to59 += _MalesAge55to59;
            this._MalesAge60to64 += _MalesAge60to64;
            this._MalesAge65to69 += _MalesAge65to69;
            this._MalesAge70to74 += _MalesAge70to74;
            this._MalesAge75to79 += _MalesAge75to79;
            this._MalesAge80AndOver += _MalesAge80AndOver;
            // Marriage Age:
            this._MalesMarriedAge0to15 += _MalesMarriedAge0to15;
            this._MalesMarriedAge16to19 += _MalesMarriedAge16to19;
            this._MalesMarriedAge20to24 += _MalesMarriedAge20to24;
            this._MalesMarriedAge25to29 += _MalesMarriedAge25to29;
            this._MalesMarriedAge30to34 += _MalesMarriedAge30to34;
            this._MalesMarriedAge35to39 += _MalesMarriedAge35to39;
            this._MalesMarriedAge40to44 += _MalesMarriedAge40to44;
            this._MalesMarriedAge45to49 += _MalesMarriedAge45to49;
            this._MalesMarriedAge50to54 += _MalesMarriedAge50to54;
            this._MalesMarriedAge55to59 += _MalesMarriedAge55to59;
            this._MalesMarriedAge60to64 += _MalesMarriedAge60to64;
            this._MalesMarriedAge65to74 += _MalesMarriedAge65to74;
            this._MalesMarriedAge75to79 += _MalesMarriedAge75to79;
            this._MalesMarriedAge80AndOver += _MalesMarriedAge80AndOver;
            // Economic Activity:
            this._MalesAge16to24Unemployed += _MalesAge16to24Unemployed;
            this._MalesAge16to74 += _MalesAge16to74;
            this._MalesAge16to74EconomicallyActiveEmployed += _MalesAge16to74EconomicallyActiveEmployed;
            this._MalesAge16to74EconomicallyActiveUnemployed += _MalesAge16to74EconomicallyActiveUnemployed;
            this._MalesAge16to74EconomicallyInactive += _MalesAge16to74EconomicallyInactive;
            this._MalesAge50AndOverUnemployed += _MalesAge50AndOverUnemployed;
        }
    }
}